use super::models;
use crate::api::core::models::Error;
use crate::core;
use std::collections::HashMap;

///
pub struct Api {
  client: reqwest::Client,
  api_config: core::config::ApiConfig,
}

impl std::fmt::Debug for Api {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Api of api.kgon.identity").finish()
  }
}

impl Api {
  ///
  pub fn new(api_config: core::config::ApiConfig) -> Api {
    Api {
      client: reqwest::Client::new(),
      api_config,
    }
  }

  ///
  pub async fn list_members(
    &self,
    data: models::ListMembersRequest,
  ) -> Result<models::ListMembersResponse, Error> {
    let mut params = HashMap::new();

    if let Some(group_key) = data.group_key {
      params.insert("groupKey", group_key);
    }

    let res = match self
      .client
      .post(format!("{}/users", self.api_config.k_url))
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.api_config.k_secret.as_str())
      .header("k-username", self.api_config.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(e) => {
        return Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::_ListMembersResponse>().await {
        Ok(r) => {
          if r.code != 0 {
            return Err(Error {
              code: r.code,
              msg: r.msg,
            });
          }

          Ok(models::ListMembersResponse { users: r.users })
        }
        Err(e) => Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        }),
      },
      _ => Err(Error {
        code: -1,
        msg: None,
      }),
    }
  }

  ///
  pub async fn create_member(
    &self,
    data: models::CreateMemberRequest,
  ) -> Result<models::CreateMemberResponse, Error> {
    let mut params = HashMap::new();

    params.insert("username", data.username);
    params.insert("nickname", data.nickname);
    params.insert("siteUsername", data.site_username);

    if let Some(group_key) = data.group_key {
      params.insert("groupKey", group_key);
    }

    let res = match self
      .client
      .post(format!("{}/register", self.api_config.k_url))
      .header(reqwest::header::ACCEPT, "application/json")
      .header(
        reqwest::header::CONTENT_TYPE,
        "application/x-www-form-urlencoded",
      )
      .header("k-secret", self.api_config.k_secret.as_str())
      .header("k-username", self.api_config.k_username.as_str())
      .form(&params)
      .send()
      .await
    {
      Ok(res) => res,
      Err(e) => {
        return Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        });
      }
    };

    match res.status() {
      reqwest::StatusCode::OK => match res.json::<models::_CreateMemberResponse>().await {
        Ok(r) => {
          if r.code != 0 {
            return Err(Error {
              code: r.code,
              msg: r.msg,
            });
          }

          match r.id {
            Some(id) => Ok(models::CreateMemberResponse { id }),
            None => Err(Error {
              code: -1,
              msg: Some("id is not exist in response of api".to_string()),
            }),
          }
        }
        Err(e) => Err(Error {
          code: -1,
          msg: Some(e.to_string()),
        }),
      },
      _ => Err(Error {
        code: -1,
        msg: None,
      }),
    }
  }
}
